{ PVCAM Demonstration Script

  This script captures a region based on a single ROI defined 
  in an image using the Region Tool on the toolbar. If the image
  does not have any ROIs defined, the entire image size is used
  as the ROI.

  NOTE: This script assumes the camera has already been opened
  using either the Camera | Install Cameras menu, or a call to the 
  pvcOpenCamera procedure. }

const
  ChooseMsg = 'Choose an image with an ROI' ;
  ErrorMsg  = 'This script requires a desktop image' ;

var
  ROI ;        { The array of ROI defs }
  ROIImage ;   { The image containing the ROI }
  NewImage ;   { The captured image }
  x1,y1 ;      { Top left of ROI }
  x2,y2 ;      { Bottom right of ROI }

begin

{ Check that a desktop image exists }
if GetNumImages = 0 then Halt( ErrorMsg ) ;

{ Select one of the images }
SelectImage( ChooseMsg,ROIImage ) ;

{ Get the ROI definitions }
ROI := GetROI( ROIImage ) ;

{ If no ROI use image dimensions }
if IsNull( ROI ) then
  begin
  x1 := 0 ;
  y1 := 0 ;
  x2 := GetXSize( ROIImage )-1 ;
  y2 := GetYSize( ROIImage )-1 ;
  end 
else
  begin
  x1 := ROI[ 0,0 ] ;
  y1 := ROI[ 1,0 ] ;
  x2 := ROI[ 2,0 ] ;
  y2 := ROI[ 3,0 ] ;
  end ;

{ Set the exposure time, gain and mode }
pvcSetExpTime( 200 ) ;
pvcSetExpMode( pvc_ExpTimed ) ;
pvcSetGain( 1 ) ;

{ Capture the an image based on the ROI }
NewImage := pvcCapture( x1,y1,x2,y2 ) ;
Show( NewImage,'pvdemo03' ) ;

end
