program Mosaic ;
{ Cut and paste regions from a source image into a new image }

{ How to use the script:

  1. Drag out selection rectangles around each region of 
     interest in the image. Remember to press the shift 
     key to define more than one region.

  2. Run the script and select the image when prompted.

  3. Choose how many columns you want. Enter zero if you 
     want the script to select a value for you.

  4. Progress messages appear on the status bar while the
     script runs and a new image "Mosaic" is created.

}

const
  { prompt messages }
  Msg1 = 'Select a source image from the list shown below' ;
  Msg2 = 'Block ' ;
  Msg3 = 'Analyzing region ' ;
  Msg4 = 'Analyzing row ' ;
  Msg5 = 'How many columns? Enter zero for automatic selection.' ;

  Err1 = 'No regions are defined in that image' ;
  Err2 = 'Out of memory' ;
  Err3 = 'There are no images loaded' ;

  Name = 'Mosaic' ;

  { other constants }
  Background = 0 ;

var
  Source ;       { source image }
  Dest ;         { destination image }
  Wide,High ;    { destination size }
  Block ;        { one image block }

  ROI ;          { list of ROIs }

  nCols,nRows ;  { number of columns and rows }
  nReg ;         { number of outlined regions }
  sRow ;         { measured row width and height }

  i,j,k,x,y ;    { indices }


procedure CalcSize( var Rgn ) ;
{ Convert region corners to origin and size }

var
  w,h ;

begin
  for k := 0 to GetYSize( Rgn ) - 1 do begin
    WriteStatus( Msg3,k ) ;
    w := Rgn[2,k] - Rgn[0,k] + 1 ;
    h := Rgn[3,k] - Rgn[1,k] + 1 ;
    Rgn[2,k] := w ;
    Rgn[3,k] := h ;
  end;
end;  { CalcSize }


function Max( a,b ) ;
{ Return the maximum variable }

begin
  if a > b then Max := a else Max := b ;
end;  { Max }


procedure Inc( var x ) ;
{ Increment a variable }

begin
  x := x + 1 ;
end;  { Inc }


{ Main program }
begin
  { choose an image }
  if GetNumImages = 0 then halt( Err3 ) ;
  if SelectImage( Msg1,Source ) <> id_Ok then halt ;

  { check for binary }
  if TypeOf( Source ) = typ_Binary then Source := byte( Source ) * 255 ;

  { get regions }
  ROI := GetROI( Source ) ;
  if IsNull( ROI ) then begin
    WriteInfo( Err1 ) ;
    halt ;
  end;

  { get number of columns }
  nReg := GetYSize( ROI ) ;
  if GetNumber( Msg5,nCols ) = id_Cancel then halt ;
  nCols := word( nCols ) ;
  if nCols = 0 then nCols := word( Ceil( Sqrt( nReg ) ) ) ;
  if nCols > nReg then nCols := nReg ;

  { calculate number of rows }
  nRows := word( Ceil( single(nReg) / nCols ) ) ;

  { calculate size of each region }
  CalcSize( ROI ) ;

  { storage for row sizes }
  sRow := CreateArray( Word,2,nRows ) ;   { 2 x N }
  if IsNull( sRow ) then halt( Err2 ) ;

  { init loop }
  k := 0 ;
  Wide := 0 ;
  High := 0 ;

  { calculate output size }
  for j := 0 to nRows-1 do begin

    { init }
    WriteStatus( Msg4,j ) ;
    sRow[0,j] := 0 ;
    sRow[1,j] := 0 ;

    { calculate width and height for row }
    for i := 0 to nCols-1 do
      if k < nReg then begin
        sRow[0,j] := sRow[0,j] + ROI[2,k] ;       { add width of each region }
        sRow[1,j] := Max( sRow[1,j],ROI[3,k] ) ;  { find max height in row }
        inc( k ) ;                                { next region }
      end;

    { calculate width and height for image }
    Wide := Max( Wide,sRow[0,j] ) ;               { find max row width }
    High := High + sRow[1,j] ;                    { add up row heights }

  end;

  { create output image }
  Dest := CreateImage( TypeOf(Source),Wide,High ) ;
  if IsNull( Dest ) then halt( Err2 ) ;

  { clear and display the image }
  Dest := Background ;
  Show( Dest,Name ) ;

  { transfer blocks }
  x := word(0) ;
  y := word(0) ;
  k := 0 ;

  for j := 0 to nRows-1 do begin

    { create an output row }
    for i := 0 to nCols-1 do
      if k < nReg then begin

        { report status }
        WriteStatus( Msg2,k ) ;

        { get a block }
        Wide  := ROI[2,k] ;
        High  := ROI[3,k] ;
        Block := GetRegion( Source,ROI[0,k],ROI[1,k],ROI[0,k]+Wide-1,ROI[1,k]+High-1 ) ;

        { move it }
        PutRegion( Dest,Block,x,y ) ;
        Update( Dest ) ;

        { increment }
        x := x + Wide ;
        inc( k ) ;

      end;

      { next row }
      x := 0 ;
      y := y + sRow[1,j] ;

  end;

end.