program Remove ;
{ Remove regions and do inverse FFT }

{ After using the dofft.v script to calculate an FFT, identify periodic
  components ("spikes") by placing flags on them or tracing regions around
  them. This script may then be used to perform a symmetric region filter
  and invert the FFT. The resulting image is rescaled and converted to byte. }

const
  Msg     = 'Choose an FFT image to process' ;
  FlagRgn = 7 ;

var
  Image ;


procedure FilterFlags( var Img; n ) ;
{ Remove symmetric regions at flags }

var
  Mask ;
  Flags ;
  i,j,k ;
  d ;
  nx,ny ;

begin
  { get flag list }
  Flags := GetFlags( Img ) ;

  { mask out flagged regions }
  if not IsNull( Flags ) then begin

    { create mask }
    Mask := CreateImage( TypeOf(img),n,n ) ;

    { get params }
    d := integer( n/2 ) ;
    nx := GetXSize( Img ) - 1 ;
    ny := GetYSize( Img ) - 1 ;

    for k := 0 to GetYSize( Flags ) - 1 do begin

      { region }
      i := Flags[0,k] - d ;
      j := Flags[1,k] - d ;
      PutRegion( Img,Mask,i,j ) ;

      { conjugate region }
      i := ( nx - Flags[0,k] ) - d ;
      j := ( ny - Flags[1,k] ) - d ;
      PutRegion( Img,Mask,i,j ) ;

    end;
  end;

end;  { FilterFlags }


procedure FilterROI( var Img ) ;
{ Remove symmetric defined regions }

var
  Mask ;
  ROI ;
  i,j,k ;
  xs,ys ;
  nx,ny ;

begin
  { get ROI list }
  ROI := GetROI( Img ) ;

  { mask out flagged regions }
  if not IsNull( ROI ) then begin

    nx := GetXSize( Img ) - 1 ;
    ny := GetYSize( Img ) - 1 ;

    for k := 0 to GetYSize( ROI ) - 1 do begin

      { region mask }
      xs := ROI[2,k] - ROI[0,k] + 1 ;
      ys := ROI[3,k] - ROI[1,k] + 1 ;
      Mask := CreateArray( TypeOf(Img),xs,ys ) ;

      { defined region }
      i := ROI[0,k] ;
      j := ROI[1,k] ;
      PutRegion( Img,Mask,i,j ) ;

      { conjugate }
      i := nx - ROI[2,k] ;
      j := ny - ROI[3,k] ;
      PutRegion( Img,Mask,i,j ) ;

      { delete }
      Free( Mask ) ;

    end;
  end;

end;  { FilterROI }


function Scale( Img ) ;
{ Scale to 0..255 }

var
  Min,Max ;

begin
  Min := MinOf( Img ) ;
  Max := MaxOf( Img ) ;
  if Max > Min then
    Scale := ( ( Img - Min ) / ( Max - Min ) ) * 255 ;
end;  { Scale }


begin
  if SelectImage( Msg,Image ) = id_Ok then begin

    { region filters }
    Beep ;
    WriteStatus( 'Filtering...' ) ;
    FilterFlags( Image,FlagRgn ) ;
    FilterROI( Image ) ;

    { clean up }
    DeleteFlags( Image ) ;
    DeleteROI( Image ) ;

    { inverse FFT }
    Beep ;
    WriteStatus( 'Calculating inverse FFT...' ) ;
    Image := real( IFFT( Image ) ) ;
    Image := byte( Scale( Image ) ) ;
    SetPalette( Image,pal_Mono ) ;

  end;
end.